// This file is auto-generated. Do not edit.
#pragma once

#include <xrpl/protocol/STTx.h>
#include <xrpl/protocol/STParsedJSON.h>
#include <xrpl/protocol/jss.h>
#include <xrpl/protocol_autogen/TransactionBase.h>
#include <xrpl/protocol_autogen/TransactionBuilderBase.h>
#include <xrpl/json/json_value.h>

#include <stdexcept>
#include <optional>

namespace xrpl::transactions {

class LoanManageBuilder;

/**
 * @brief Transaction: LoanManage
 *
 * Type: ttLOAN_MANAGE (82)
 * Delegable: Delegation::notDelegable
 * Amendment: featureLendingProtocol
 * Privileges: mayModifyVault
 *
 * Immutable wrapper around STTx providing type-safe field access.
 * Use LoanManageBuilder to construct new transactions.
 */
class LoanManage : public TransactionBase
{
public:
    static constexpr xrpl::TxType txType = ttLOAN_MANAGE;

    /**
     * @brief Construct a LoanManage transaction wrapper from an existing STTx object.
     * @throws std::runtime_error if the transaction type doesn't match.
     */
    explicit LoanManage(std::shared_ptr<STTx const> tx)
        : TransactionBase(std::move(tx))
    {
        // Verify transaction type
        if (tx_->getTxnType() != txType)
        {
            throw std::runtime_error("Invalid transaction type for LoanManage");
        }
    }

    // Transaction-specific field getters

    /**
     * @brief Get sfLoanID (soeREQUIRED)
     * @return The field value.
     */
    [[nodiscard]]
    SF_UINT256::type::value_type
    getLoanID() const
    {
        return this->tx_->at(sfLoanID);
    }
};

/**
 * @brief Builder for LoanManage transactions.
 *
 * Provides a fluent interface for constructing transactions with method chaining.
 * Uses Json::Value internally for flexible transaction construction.
 * Inherits common field setters from TransactionBuilderBase.
 */
class LoanManageBuilder : public TransactionBuilderBase<LoanManageBuilder>
{
public:
    /**
     * @brief Construct a new LoanManageBuilder with required fields.
     * @param account The account initiating the transaction.
     * @param loanID The sfLoanID field value.
     * @param sequence Optional sequence number for the transaction.
     * @param fee Optional fee for the transaction.
     */
    LoanManageBuilder(SF_ACCOUNT::type::value_type account,
                     std::decay_t<typename SF_UINT256::type::value_type> const& loanID,                    std::optional<SF_UINT32::type::value_type> sequence = std::nullopt,
                    std::optional<SF_AMOUNT::type::value_type> fee = std::nullopt
)
        : TransactionBuilderBase<LoanManageBuilder>(ttLOAN_MANAGE, account, sequence, fee)
    {
        setLoanID(loanID);
    }

    /**
     * @brief Construct a LoanManageBuilder from an existing STTx object.
     * @param tx The existing transaction to copy from.
     * @throws std::runtime_error if the transaction type doesn't match.
     */
    LoanManageBuilder(std::shared_ptr<STTx const> tx)
    {
        if (tx->getTxnType() != ttLOAN_MANAGE)
        {
            throw std::runtime_error("Invalid transaction type for LoanManageBuilder");
        }
        object_ = *tx;
    }

    /** @brief Transaction-specific field setters */

    /**
     * @brief Set sfLoanID (soeREQUIRED)
     * @return Reference to this builder for method chaining.
     */
    LoanManageBuilder&
    setLoanID(std::decay_t<typename SF_UINT256::type::value_type> const& value)
    {
        object_[sfLoanID] = value;
        return *this;
    }

    /**
     * @brief Build and return the LoanManage wrapper.
     * @param publicKey The public key for signing.
     * @param secretKey The secret key for signing.
     * @return The constructed transaction wrapper.
     */
    LoanManage
    build(PublicKey const& publicKey, SecretKey const& secretKey)
    {
        sign(publicKey, secretKey);
        return LoanManage{std::make_shared<STTx>(std::move(object_))};
    }
};

}  // namespace xrpl::transactions
